// Usage:
//
// these files should be in one folder:
//	doubleblind_index.html		(view this in a web browser)
//	doubleblind_javascript.js
//	0.wav
//	1.wav
//
//
//
// the webpage should also use this type of HTML for the buttons etc. that this script uses ..
/**
<body onload="webpage_init();">
//		this ensures the method gets called whenever webpage is loaded ..

...


<p>
<b id="trial_count">Starting Trial (1/10):</b>
</p>




<H1 id="layout_downloading">
<pre>
	Downloading...
</pre>
</H1>




<p id="layout_trial">

<b>Listen: </b>
<button onclick="button_listen(sample0_FileIndex)">Sample 0</button>
<button onclick="button_listen(sample1_FileIndex)">Sample 1</button>
<i>    </i>
<i id="audio_output_state"></i>



<br>
<br>


<b>Vote for Hi-Res:</b>
<button onclick="button_vote(sample0_FileIndex, 0)">Sample 0 was Hi-Res</button>
<button onclick="button_vote(sample1_FileIndex, 0)">Sample 1 was Hi-Res</button>



<br>
<br>



<b>Results: </b>

<br>
<i id="result_output">...</i>

</p>




...


</body>

*/





// HTML:
//	simple HTML refresher:
//	https://www.lehigh.edu/~inwww/seminar/reference/htmlchart.html

// Javascript Audio:
//
// double-blind test inspired by: http://www.audiocheck.net/blindtests_16vs8bit_NeilYoung.php
//	which uses javascript to show 16-bit and 8-bit samples
//
// https://developer.mozilla.org/en-US/docs/Web/API/HTMLAudioElement
// https://stackoverflow.com/questions/9419263/playing-audio-with-javascript
//	Firefox won't play MP3s, will play OGG and WAV
//	IE won't play WAV
// http://www.storiesinflight.com/html5/audio.html
//
// for better control of audio .. use:
//	https://github.com/goldfire/howler.js
//	https://howlerjs.com
//
// https://www.html5rocks.com/en/tutorials/webaudio/intro/
// https://webaudio.github.io/web-audio-api/
//
// https://stackoverflow.com/questions/11506180/web-audio-api-resume-from-pause
//




// use this to set if want trials to reset trial count after a certain number of trials ..
//	NOTE: user can also reload webpage to reset the trial count
//
//const USE_TRIAL_MAXNUM = true;
const USE_TRIAL_MAXNUM = false;
const TRIAL_MAXNUM = 10;

// will use 0.wav and 1.wav
//
const FILE_EXT = ".wav";

// NOTE: if want buttons to do Play/Stop .. then just comment this out:
// setting
//const ENABLE_RESUME = true;
const ENABLE_RESUME = false;

//const DEBUG_FLAG = true;
const DEBUG_FLAG = false;








// ASSERTION: 0.wav should be the Hi-Res version, and 1.wav the Low-Res version
//
// whether Sample 0 and Sample 1 are:
//	0.wav, 1.wav
//	or
//	1.wav, 0.wav
//
// if one is 0, the other will be 1
//	button_randomize() randomizes these again ..
//	these indicate the samples which associated with the buttons ..
//
// setting this to 0, 1 initially, but these will get randomized in:
//	webpage_init()
//		which calls:
//		button_randomize();
//
var sample0_FileIndex = 0;
var sample1_FileIndex = 1;

var trial_count = 0;
var trial_correct_count = 0;



// do it this way so starts downloading ..
//var object_audio_object = new Audio();
var object_audio_Array = [new Audio('0.wav'), new Audio('1.wav')];

// will allow playing .. only when both 0.wav and 1.wav can be played ..
var object_audio_canplayFlag = [false, false];



//const PLAYSTATE_STOPPED = "Stopped";
// just erase .. don't show text ..
const PLAYSTATE_STOPPED = " ";
const PLAYSTATE_PLAYING = "Playing";
const PLAYSTATE_PAUSED = "Paused";
var object_audio_state = PLAYSTATE_STOPPED;
var object_audio_currentTimeIfPaused = 0;
// if playing .. which one is playing (affects behavior of other buttons .. if they pause/stop .. or they stop other and then start playing)
var object_audio_playingFileIndex = sample0_FileIndex;




function webpage_init() {
	layout_update();

	audio_downloadfiles();

	button_randomize();
	audio_initFile(0);
	audio_initFile(1);

    	print_trial_count();
}
function print_trial_count() {
	if (!USE_TRIAL_MAXNUM) {
		// not show maxnum
    		document.getElementById("trial_count").innerHTML = "Starting Trial " + (trial_count + 1) + ":";
	} else {
		// use TRIAL_MAXNUM
    		document.getElementById("trial_count").innerHTML = "Starting Trial (" + (trial_count + 1) + "/" + TRIAL_MAXNUM + "):";
	}
}
function print_trial_result(temp_correctFlag) {
	var success_percentage = Math.floor(100 * trial_correct_count / trial_count); 
	var confidence_interval_string = "";

	var sig = sigma();

    	if ((trial_count > 9) && (sig >= 95) && (success_percentage > 80)) {
		confidence_interval_string = "Confidence: " + sig + "%" + " - Passed the test!";
	} else {
		confidence_interval_string = "Confidence: " + sig + "%";
	}

	if (temp_correctFlag) {
		document.getElementById("result_output").innerHTML = "Correct - Current score " + trial_correct_count + "/" + trial_count + " (" + success_percentage + "%) - " + confidence_interval_string;
	} else {
		document.getElementById("result_output").innerHTML = "Wrong - Current score " + trial_correct_count + "/" + trial_count + " (" + success_percentage + "%) - " + confidence_interval_string;
	}
}


// confidence interval code copied from:
//	http://www.audiocheck.net/blindtests_16vs8bit_NeilYoung.php
//
var f = [];

function fac(n) {
    if (n == 0 || n == 1)
        return 1;
    if (f[n] > 0)
        return f[n];
    return f[n] = fac(n - 1) * n;
}
function sigma() {
    //var p = 1 / allSoundIDs.length;
    var p = 1 / 2;	// Sample 0 and Sample 1 = 2 samples
    var sig = 1;

    for (var k = trial_correct_count; k <= trial_count; k++) sig -= (fac(trial_count) / (fac(k) * fac(trial_count - k))) * Math.pow(p, k) * Math.pow(1 - p, trial_count - k);

    sig = Math.max(0, Math.floor(10000 * sig) / 100);

    return sig;
}





// show "layout_downloading" if not have "canplaythrough" true for both 0.wav and 1.wav ..
// hide "layout_downloading"/show "layout_trial" .. when "canplaythrough" both ..
//
//
// https://stackoverflow.com/questions/6242976/javascript-hide-show-element
//
// make gone (so not take up space on screen)
//	similar to android:visibility="gone"
function ui_element_gone(temp_id) {
	// remove element
	document.getElementById(temp_id).style.display = 'none';
	//
	// visible
	//document.getElementById(temp_id).style.display = '';
	// doesn't work
	//document.getElementById(temp_id).style.display = 'block';
}
// make invisible but still taking up space on screen
function ui_element_invisible(temp_id) {
	// invisible (element still takes up space
	document.getElementById(temp_id).style.visibility = 'hidden';
	//
	// visible
	//document.getElementById(temp_id).style.visibility = 'visible';
}
// make visible
function ui_element_show(temp_id) {
	document.getElementById(temp_id).style.display = '';
}

function layout_update() {

	//ui_element_show('layout_downloading');
	//ui_element_show('layout_trial');

	if (object_audio_canplayFlag[0] == true && object_audio_canplayFlag[1] == true) {
		// can play both
		//	show trial ..
		ui_element_gone('layout_downloading');
		ui_element_show('layout_trial');
	} else {
		// cannot play yet
		//	show downloading ..
		ui_element_show('layout_downloading');
		ui_element_gone('layout_trial');
	}

}









function audio_downloadfiles() {
    // https://www.w3schools.com/js/js_window_location.asp

    //window.location + '/0.wav';
    //window.location + '/1.wav';
}
function audio_initFile(temp_fileIndex) {

    // temp_fileIndex value is passed as constant down to the inner function (like inner classes ..
    const inner_fileIndex = temp_fileIndex;


    // set up listeners

    // https://www.w3schools.com/tags/av_event_play.asp
    object_audio_Array[inner_fileIndex].onplay = function() {
    		// are playing now ..
		//----------
    		object_audio_state = PLAYSTATE_PLAYING;

		// show on screen
    		//document.getElementById("audio_output_state").innerHTML = object_audio_state + " file:" + inner_fileIndex;
	    	if (inner_fileIndex == sample0_FileIndex) {
    			document.getElementById("audio_output_state").innerHTML = object_audio_state + " Sample 0";
		} else if (inner_fileIndex == sample1_FileIndex) {
    			document.getElementById("audio_output_state").innerHTML = object_audio_state + " Sample 1";
		} else {
			// shouldn't happen
    			document.getElementById("audio_output_state").innerHTML = object_audio_state + " Undefined";
		}
/**
		if (object_audio_playingFileIndex == sample0_FileIndex) {
    			document.getElementById("audio_output_state").innerHTML = object_audio_state + " Sample 0";
		} else {
			//object_audio_playingFileIndex == sample1_FileIndex
    			document.getElementById("audio_output_state").innerHTML = object_audio_state + " Sample 1";
		}
*/
		//----------
    };
    object_audio_Array[inner_fileIndex].onended = function() {
	        // playing has ended ..
		//----------
		object_audio_state = PLAYSTATE_STOPPED;

		// show on screen
    		document.getElementById("audio_output_state").innerHTML = object_audio_state;
		//----------
    };
    object_audio_Array[inner_fileIndex].onpause = function() {

		// ASSERTION: since stop() doesn't work, are going to use pause() to do that also .. (going to use position = 0.0 as indication of stop)
		// since can't trigger stop listener ..
		//
		// this will indicate to the pause listener .. to behave like stop listener ..
		//
		//	https://www.w3schools.com/JSREF/dom_obj_audio.asp
		//	currentTime	Sets or returns the current playback position in an audio (in seconds)

	    	if (object_audio_currentTimeIfPaused == 0.0) {
			// ASSERTION: since stop() doesn't work, are going to use pause() to do that also .. (going to use position = 0.0 as indication of stop)
			// paused at 0.0 position .. treat this as a stop ..

			// do what are doing for stop listener above ..
			//----------
			object_audio_state = PLAYSTATE_STOPPED;
			// show on screen
			document.getElementById("audio_output_state").innerHTML = object_audio_state;
			//----------
		} else {
	    		// object_audio_currentTimeIfPaused != 0.0
			
			// ASSERTION: since stop() doesn't work, are going to use pause() to do that also .. (going to use position = 0.0 as indication of stop)
			// pause listener

			//----------
			object_audio_state = PLAYSTATE_PAUSED;
			// show on screen
			//document.getElementById("audio_output_state").innerHTML = object_audio_state;

	    		if (inner_fileIndex == sample0_FileIndex) {
    				document.getElementById("audio_output_state").innerHTML = object_audio_state + " Sample 0";
			} else if (inner_fileIndex == sample1_FileIndex) {
    				document.getElementById("audio_output_state").innerHTML = object_audio_state + " Sample 1";
			} else {
				// shouldn't happen
    				document.getElementById("audio_output_state").innerHTML = object_audio_state + " Undefined";
			}

/**
			if (object_audio_playingFileIndex == sample0_FileIndex) {
				document.getElementById("audio_output_state").innerHTML = object_audio_state + " Sample 0";
			} else {
				//object_audio_playingFileIndex == sample1_FileIndex
				document.getElementById("audio_output_state").innerHTML = object_audio_state + " Sample 1";
			}
*/

			// remember this - so can resume from there
			//	https://www.w3schools.com/JSREF/dom_obj_audio.asp
			//	currentTime	Sets or returns the current playback position in an audio (in seconds)
			object_audio_currentTimeIfPaused = object_audio_Array[inner_fileIndex].currentTime;
			//----------
		}

    };
    // https://developer.mozilla.org/en-US/docs/Web/Events/canplay
    //	The canplay event is fired when the user agent can play the media, but estimates that not enough data has been loaded
    //	to play the media up to its end without having to stop for further buffering of content.
    //
    object_audio_Array[inner_fileIndex].oncanplay = function() {
	    	// can start playing .. does not mean it's playing ..
	        //	"canplay" not as good as indicator of loading .. as "canplaythrough" which estimates that can play whole file without buffering ..
	        //
		//	do nothing in "canplay" .. but do in "canplaythrough" ..
		//----------
    		if (DEBUG_FLAG) {
		    // show on screen
    		    //document.getElementById("audio_output_state").innerHTML = "canplay file: " + inner_fileIndex;
		}
		//----------
    };
    // https://developer.mozilla.org/en-US/docs/Web/Events/canplaythrough
    //	The canplaythrough event is fired when the user agent can play the media, and estimates that enough data has been loaded
    //	to play the media up to its end without having to stop for further buffering of content.
    //
    // this will inform when the file is available for play (i.e. have enough that will play while other streams)
    //		ok, in practice this happens .. but still the audio not plays .. i.e. it can overestimate the smoothness with which it will play ..
    //		should also be a listener for if have full ..
    //
    //	so "canplaythrough" is a better indicator (than "canplay") that will be able to play whole file without issue (though not guaranteed .. is an estimate)
    //		while "canplay" only says that can start playing .. but don't have estimate whether can play whole file without issue ..

    // either of these work ..
    if (true) {

	    object_audio_Array[inner_fileIndex].oncanplaythrough = function() {
	    	    // can start playing .. does not mean it's playing ..
	            //	"canplay" not as good as indicator of loading .. as "canplaythrough" which estimates that can play whole file without buffering ..
	            //
		    //	do nothing in "canplay" .. but do in "canplaythrough" ..
		    //
		    //alert("Can play through audio without stopping");

		    // set flags that is able to play ..
		    //	and to make buttons visible ..
		    //----------
	    	    object_audio_canplayFlag[inner_fileIndex] = true;

	    	    // since something changed .. can check now if both are now available
	    	    if (object_audio_canplayFlag[0] == true && object_audio_canplayFlag[1] == true) {
			// have both now .. can play .. so show the trial buttons etc.
			layout_update();
		    }
		    //----------


    		    if (DEBUG_FLAG) {
		    	// show on screen
    			//document.getElementById("audio_output_state").innerHTML = "canplaythrough file: " + inner_fileIndex;
		    	document.getElementById("audio_output_state").innerHTML = "canplaythrough status: " + object_audio_canplayFlag[0] + " " + object_audio_canplayFlag[1];
		    }
	    };
    } else {
	    object_audio_Array[inner_fileIndex].addEventListener('canplaythrough', function() {
	    	    // can start playing .. does not mean it's playing ..
	            //	"canplay" not as good as indicator of loading .. as "canplaythrough" which estimates that can play whole file without buffering ..
	            //
		    //	do nothing in "canplay" .. but do in "canplaythrough" ..
		    //
		    //alert("Can play through audio without stopping");

		    // set flags that is able to play ..
		    //	and to make buttons visible ..
		    //----------
	    	    object_audio_canplayFlag[inner_fileIndex] = true;

	    	    // since something changed .. can check now if both are now available
	    	    if (object_audio_canplayFlag[0] == true && object_audio_canplayFlag[1] == true) {
			// have both now .. can play .. so show the trial buttons etc.
			layout_update();
		    }
		    //----------


    		    if (DEBUG_FLAG) {
		    	// show on screen
    			//document.getElementById("audio_output_state").innerHTML = "canplaythrough file: " + inner_fileIndex;
		    	document.getElementById("audio_output_state").innerHTML = "canplaythrough status: " + object_audio_canplayFlag[0] + " " + object_audio_canplayFlag[1];
		    }
	    });
    }

    // http://softwareas.com/audiovideo-tag-dont-forget-to-load-before-you-play/
    //
    // this will start downloading ..
    // this will trigger the "canplaythrough" listener eventually ..
    object_audio_Array[inner_fileIndex].load();


    object_audio_Array[inner_fileIndex].loop = 0;
    object_audio_Array[inner_fileIndex].volume = 1;
}

function button_randomize() {
    // assign random initially
    var num = 2;
    var temp_fileIndex = Math.floor(Math.random() * num);
    if (temp_fileIndex == 0) {
    	sample0_FileIndex = 0;
	// set it to the other one
        sample1_FileIndex = 1;
    } else {
    	sample0_FileIndex = 1;
	// set it to the other one
        sample1_FileIndex = 0;
    }
}

function play_pause() {
    // ASSERTION: since stop() doesn't work, are going to use pause() to do that also .. (going to use position = 0.0 as indication of stop)
    // ensure this is not zero .. so not confused with a pause that using to stop ..
    object_audio_currentTimeIfPaused = 0.00001;

    // pause any playing - not do any more
    if (!isNaN(object_audio_Array[0].duration)) {
        object_audio_Array[0].pause();
    }
    if (!isNaN(object_audio_Array[1].duration)) {
        object_audio_Array[1].pause();
    }
}
// ASSERTION: since stop() doesn't work, are going to use pause() to do that also .. (going to use position = 0.0 as indication of stop)
function play_stop() {

    // ASSERTION: since stop() doesn't work, are going to use pause() to do that also .. (going to use position = 0.0 as indication of stop)
    // since can't trigger stop listener ..
    //
    // this will indicate to the pause listener .. to behave like stop listener ..
    //
    //	https://www.w3schools.com/JSREF/dom_obj_audio.asp
    //	currentTime	Sets or returns the current playback position in an audio (in seconds)
    object_audio_currentTimeIfPaused = 0.0;

    // pause any playing - not do any more
    if (!isNaN(object_audio_Array[0].duration)) {
        object_audio_Array[0].pause();
    }
    if (!isNaN(object_audio_Array[1].duration)) {
        object_audio_Array[1].pause();
    }
}
// which button is playing .. and play from start (0.0) or from positionInSecondsFloat
function play_start(temp_fileIndex, temp_positionInSecondsFloat) {

    // remember which button's file is playing ..
    object_audio_playingFileIndex = temp_fileIndex;


    // start playing file constructed using temp_fileIndex
    //		0.wav
    //		1.wav
    //object_audio_Array[0].src = temp_fileIndex + FILE_EXT;
    //object_audio_Array[1].src = temp_fileIndex + FILE_EXT;
    

    // pause any playing - not do any more
    if (!isNaN(object_audio_Array[0].duration)) {
        object_audio_Array[0].pause();
    }
    if (!isNaN(object_audio_Array[1].duration)) {
        object_audio_Array[1].pause();
    }

    // play - seeking forward from where previously paused ..
    object_audio_Array[temp_fileIndex].play();
    //		need to do this after play() - else doesn't seek forward ..
    object_audio_Array[temp_fileIndex].currentTime = temp_positionInSecondsFloat;
}


// temp_fileIndex specifies the file index to use i.e. 0.wav or 1.wav ..
function play_pressed(temp_fileIndex) {

    if (object_audio_state == PLAYSTATE_PLAYING) {

	// behavior depends on which button pressed ..
	//	pause/stop (depending on ENABLE_RESUME) if same button
	//	stop and play the new button if different button

	if (temp_fileIndex == object_audio_playingFileIndex) {
		if (ENABLE_RESUME) {
			// same as this button (or referring to same file) .. so pause
			// pause
			play_pause();
			//----------
			return;
			//----------
		} else {
			// same as this button (or referring to same file) .. so stop

			// ASSERTION: since stop() doesn't work, are going to use pause() to do that also .. (going to use position = 0.0 as indication of stop)
			play_stop();
			//----------
			return;
			//----------
		}
	} else {
		// different from this button (or referring to different file) .. so stop and then play the new one ..

		// play from the start
		var temp_positionInSecondsFloat = 0.0;
		play_start(temp_fileIndex, temp_positionInSecondsFloat);

		//----------
		return;
		//----------
	}

    } else if (object_audio_state == PLAYSTATE_STOPPED) {
        // stop current playing button - and play the button that has been pressed ..

	// play from the start
    	var temp_positionInSecondsFloat = 0.0;
    	play_start(temp_fileIndex, temp_positionInSecondsFloat);

    } else if (object_audio_state == PLAYSTATE_PAUSED) {

	// behavior depends on which button pressed ..
	//	resume if same button
	//	stop and play the new button if different button

	if (temp_fileIndex == object_audio_playingFileIndex) {
		// same as this button (or referring to same file) .. so pause should be resumed

		// resume
		// if was paused, then "object_audio_currentTimeIfPaused" should be set - will play (resume) from that point on .. - will play below

		// want to resume .. from this remembered position ..
		//var temp_positionInSecondsFloat = 0.0;
		var temp_positionInSecondsFloat = object_audio_currentTimeIfPaused;
		play_start(temp_fileIndex, temp_positionInSecondsFloat);

		//----------
		return;
		//----------
	} else {
		// different from this button (or referring to different file) .. so stop and then play the new one ..

		// play from the start
		var temp_positionInSecondsFloat = 0.0;
		play_start(temp_fileIndex, temp_positionInSecondsFloat);

		//----------
		return;
		//----------
	}

    } else {
        // do nothing
	//----------
        return;
	//----------
    }



}
function button_listen(temp_fileIndex) {

    if (DEBUG_FLAG) {
	    //document.getElementById("result_output").innerHTML = Date();
	    document.getElementById("result_output").innerHTML = temp_fileIndex;
    }
    
    play_pressed(temp_fileIndex);
}

// user votes with this .. i.e. which button has the 0.wav (or index == 0)
//	temp_matchThisIndex == 0 .. if trying to find the highest resolution
function button_vote(temp_fileIndex, temp_matchThisIndex) {

    // stop the playing ..
    play_stop();


    trial_count++;
    
    if (temp_fileIndex == temp_matchThisIndex) {
        // chose correctly
        trial_correct_count++;

    	print_trial_result(true);
    } else {
    	// chose something else ..
        //		chose incorrectly

    	print_trial_result(false);
    }



    // reset trial count (if there is a TRIAL_MAXNUM)
    if (USE_TRIAL_MAXNUM) {
	    if (trial_count >= TRIAL_MAXNUM) {
		trial_count = 0;
		trial_correct_count = 0;
	    }
    }
    print_trial_count();

    // reset
    //		since user has chosen result
    //		and we will now trigger new experiment
    button_randomize();
}






